/* misc.c -- miscellaneous library-like routines for fsgrab
 *
 * Aaron Crane <aaronc@pobox.com>
 *
 * This file is part of `e2recover', a suite of programs which assist in
 * recovering deleted files from ext2 file systems.  `e2recover' is
 * Copyright  1997, 1998, 1999 Aaron Crane <aaronc@pobox.com>.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <misc.h>

#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <ctype.h>

const char *invocation_name = "";

void *
mustmalloc (size_t n)
{
    void *p = malloc (n);

    if (!p)
	fatal ("out of memory\n");
    return p;
}

inline static void
vmessage (const char *fmt, va_list va)
{
    fprintf (stderr, "%s: ", invocation_name);
    vfprintf (stderr, fmt, va);
}

void
message (const char *fmt, ...)
{
    va_list va;

    va_start (va, fmt);
    vmessage (fmt, va);
    va_end (va);
}

void
fataln (int status, const char *fmt, ...)
{
    va_list va;

    va_start (va, fmt);
    vmessage (fmt, va);
    va_end (va);
    if (status >= 0)
	exit (status);
    abort ();
}

void
fatal (const char *fmt, ...)
{
    va_list va;

    va_start (va, fmt);
    vmessage (fmt, va);
    va_end (va);
    exit (1);
}

static unsigned long
multiplier (char c)
{
    switch (tolower ((unsigned char) c))
    {
    case 'w':
	return 2;
    case 'b':
	return 512;
    case 'k':
	return 1024;
    case 'm':
	return 1024uL * 1024uL;
    default:
	return 0;
    }
}

inline static void
val_overflow (const char *s)
{
    message ("value too large: %s\n", s);
    short_usage ();
}

size_t
safe_atosz (const char *s, int allow_multiplier)
{
    char *endp;
    unsigned long ul = 0;
    size_t sz;

    if (*s == 0)
	short_usage ();		/* don't allow empty numbers */
    errno = 0;			/* so we can reliably detect overflow */
    ul = strtoul (s, &endp, 0);
    if (*endp != 0)		/* some trailing characters weren't converted */
    {
	unsigned long mul;
	if (allow_multiplier && endp[1] == 0 && (mul = multiplier (*endp)) > 0)
	    ul *= mul;
	else
	{
	    message ("garbage in numerical value: %s\n", s);
	    short_usage ();
	}
    }

    if (ul == ULONG_MAX && errno == ERANGE)
	val_overflow (s);

    sz = ul;
    if (sz != ul)
	val_overflow (s);

    return sz;
}
